--
-- HonkTrigger
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2015-06-14 - Initial implementation
-- 

HonkTrigger = {};

local HonkTrigger_mt = Class(HonkTrigger, Object);
InitObjectClass(HonkTrigger, "HonkTrigger");

function HonkTrigger.onCreate(id)
	local instance = HonkTrigger:new(g_server ~= nil, g_client ~= nil);
	if instance:load(id) then
		g_currentMission:addOnCreateLoadedObject(instance);
		instance:register(true);
	else
		instance:delete();
	end;
end;
g_onCreateUtil.addOnCreateFunction("HonkTrigger", HonkTrigger.onCreate);

function HonkTrigger:new(isServer, isClient)
    local self = Object:new(isServer, isClient, HonkTrigger_mt);
    self.className = "HonkTrigger";
	
	self.triggerId = 0;
	
    return self;
end;

function HonkTrigger:load(id)
	self.triggerId = id;
	self.isEnabled = true;
	addTrigger(self.triggerId, "triggerCallback", self);
	
	self.numInTrigger = 0;
	self.inTrigger = {};
	
	self.activateHonk = false;
	
	self.honkPaused = false;
	self.honkActive = false;
	self.honkTimes = {};
	self.honkTimes["CURRENT"] = 0;
	self.honkTimes["MAX"] = Utils.getNoNil(getUserAttribute(id, "maxHonkTime"), 1500);
	self.honkTimes["PAUSE"] = Utils.getNoNil(getUserAttribute(id, "pauseTime"), 1000);
	
	if self.isClient then
		local outerRadius = Utils.getNoNil(getUserAttribute(id, "outerRadius"), 75);
		local innerRadius = Utils.getNoNil(getUserAttribute(id, "innerRadius"), 35);
		local soundFilename = getUserAttribute(id, "soundFilename");
		if soundFilename == nil then
			soundFilename = "$data/vehicles/shared/hornBig.wav";
		end;
		if soundFilename ~= "" and soundFilename ~= "none" then
			soundFilename = Utils.getFilename(soundFilename, g_currentMission.baseDirectory);
			self.honkSound = createAudioSource("HonkTrigger", soundFilename, outerRadius, innerRadius, 1, 0);
			link(id, self.honkSound);
			setVisibility(self.honkSound, false);
		end;
	end;
	
	return true;
end;

function HonkTrigger:delete()
	self.inTrigger = {};
	self.numInTrigger = 0;
	
	if self.isClient then
		if self.honkSound ~= nil then
			delete(self.honkSound);
		end;
	end;
	
	if self.triggerId ~= 0 then
		removeTrigger(self.triggerId);
	end;
end;

function HonkTrigger:readStream(streamId, connection)
end;

function HonkTrigger:writeStream(streamId, connection)
end;

function HonkTrigger:update(dt)
	if self.activateHonk then
		if self.honkTimes["CURRENT"] == 0 and not self.honkPaused then
			self.honkTimes["CURRENT"] = g_currentMission.time + self.honkTimes["MAX"];
		end;
		
		if self.honkTimes["CURRENT"] > g_currentMission.time then
			if not self.honkPaused then
				self:startSound();
			end;
		else
			if not self.honkPaused then
				self:stopSound();
				self.honkTimes["CURRENT"] = g_currentMission.time + self.honkTimes["PAUSE"];
				self.honkPaused = true;
			else
				self.honkPaused = false;
				self.honkTimes["CURRENT"] = 0;
			end;
		end;
		
	else
		if self.honkTimes["CURRENT"] < g_currentMission.time then
			self:stopSound();
			self.honkTimes["CURRENT"] = 0;
		end;
	end;
end;

function HonkTrigger:startSound()
	if self.isClient and self.honkSound ~= nil then
		if not self.honkActive then
			setVisibility(self.honkSound, true);
			self.honkActive = true;
		end;
	end;
end;

function HonkTrigger:stopSound()
	if self.isClient and self.honkSound ~= nil then
		if self.honkActive then
			setVisibility(self.honkSound, false);
			self.honkActive = false;
		end;
	end;
end;

function HonkTrigger:triggerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
	if self.isEnabled then
		local vehicle = g_currentMission.nodeToVehicle[otherId];
		if vehicle ~= nil then
			if onLeave then
				if self.inTrigger[otherId] ~= nil then
					self.inTrigger[otherId] = nil;
					self.numInTrigger = self.numInTrigger - 1;
					
					if self.numInTrigger <= 0 then
						self.activateHonk = false;
						self.numInTrigger = 0;
					end;
				end;
			else
				if self.inTrigger[otherId] == nil then
					self.inTrigger[otherId] = true;
					self.activateHonk = true;
					self.numInTrigger = self.numInTrigger + 1;
				end;
			end;
		end;
	end;
end;